# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grid/v1/experiment.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from grid.protos.grid.v1 import metadata_pb2 as grid_dot_v1_dot_metadata__pb2
from grid.protos.grid.v1 import datastore_pb2 as grid_dot_v1_dot_datastore__pb2
from grid.protos.grid.v1 import image_pb2 as grid_dot_v1_dot_image__pb2
from grid.protos.grid.v1 import resource_pb2 as grid_dot_v1_dot_resource__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='grid/v1/experiment.proto',
  package='grid.v1',
  syntax='proto3',
  serialized_options=b'Z0github.com/gridai/grid/grid-backend/apis/grid/v1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x18grid/v1/experiment.proto\x12\x07grid.v1\x1a\x16grid/v1/metadata.proto\x1a\x17grid/v1/datastore.proto\x1a\x13grid/v1/image.proto\x1a\x16grid/v1/resource.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x19google/protobuf/any.proto\"\x9b\x01\n\nExperiment\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x11.grid.v1.MetadataR\x08metadata\x12+\n\x04spec\x18\x02 \x01(\x0b\x32\x17.grid.v1.ExperimentSpecR\x04spec\x12\x31\n\x06status\x18\x03 \x01(\x0b\x32\x19.grid.v1.ExperimentStatusR\x06status\"F\n\x0cScratchSpace\x12\x17\n\x07size_gb\x18\x01 \x01(\rR\x06sizeGb\x12\x1d\n\nmount_path\x18\x02 \x01(\tR\tmountPath\"\xe4\x05\n\x0e\x45xperimentSpec\x12\x32\n\x03\x65nv\x18\x01 \x03(\x0b\x32 .grid.v1.ExperimentSpec.EnvEntryR\x03\x65nv\x12\x37\n\x08\x65xecutor\x18\x02 \x01(\x0b\x32\x1b.grid.v1.ExperimentExecutorR\x08\x65xecutor\x12=\n\rdesired_state\x18\x04 \x01(\x0e\x32\x18.grid.v1.ExperimentStateR\x0c\x64\x65siredState\x12\x15\n\x06run_id\x18\x06 \x01(\tR\x05runId\x12\x17\n\x07user_id\x18\x07 \x01(\tR\x06userId\x12>\n\x0bsource_code\x18\x08 \x01(\x0b\x32\x1d.grid.v1.ExperimentSourceCodeR\nsourceCode\x12\x18\n\x07\x63ommand\x18\t \x03(\tR\x07\x63ommand\x12\x30\n\x14relative_working_dir\x18\n \x01(\tR\x12relativeWorkingDir\x12\x30\n\tresources\x18\x0b \x01(\x0b\x32\x12.grid.v1.ResourcesR\tresources\x12/\n\x06inputs\x18\x0c \x03(\x0b\x32\x17.grid.v1.DatastoreInputR\x06inputs\x12)\n\x10pause_automation\x18\r \x01(\x08R\x0fpauseAutomation\x12/\n\x07scratch\x18\x0e \x03(\x0b\x32\x15.grid.v1.ScratchSpaceR\x07scratch\x12\x1c\n\tframework\x18\x0f \x01(\tR\tframework\x12\x37\n\x08strategy\x18\x10 \x01(\x0e\x32\x1b.grid.v1.ExperimentStrategyR\x08strategy\x12\x1c\n\tresumable\x18\x11 \x01(\x08R\tresumable\x1a\x36\n\x08\x45nvEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xb2\x02\n\x12\x45xperimentExecutor\x12\x1d\n\ncluster_id\x18\x01 \x01(\tR\tclusterId\x12\x32\n\x08\x65xternal\x18\x02 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00R\x08\x65xternal\x12G\n\nkubernetes\x18\x03 \x01(\x0b\x32%.grid.v1.ExperimentKubernetesExecutorH\x00R\nkubernetes\x12\x38\n\x05slurm\x18\x04 \x01(\x0b\x32 .grid.v1.ExperimentSLURMExecutorH\x00R\x05slurm\x12:\n\tscheduler\x18\x05 \x01(\x0b\x32\x1c.grid.v1.ExperimentSchedulerR\tschedulerB\n\n\x08\x65xecutor\"\x94\x01\n\x13\x45xperimentScheduler\x12\x32\n\x08\x65xternal\x18\x01 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00R\x08\x65xternal\x12<\n\x06simple\x18\x02 \x01(\x0b\x32\".grid.v1.SimpleExperimentSchedulerH\x00R\x06simpleB\x0b\n\tscheduler\"J\n\x19SimpleExperimentScheduler\x12-\n\x12\x63\x61ndidate_clusters\x18\x01 \x03(\tR\x11\x63\x61ndidateClusters\"(\n\x14\x45xperimentSourceCode\x12\x10\n\x03\x64sn\x18\x01 \x01(\tR\x03\x64sn\"\x88\x01\n\x1c\x45xperimentKubernetesExecutor\x12#\n\rinstance_type\x18\x01 \x01(\tR\x0cinstanceType\x12\x19\n\x08use_spot\x18\x02 \x01(\x08R\x07useSpot\x12(\n\x05image\x18\x03 \x01(\x0b\x32\x12.grid.v1.ImageSpecR\x05image\"\x19\n\x17\x45xperimentSLURMExecutor\"\xb6\x03\n\x10\x45xperimentStatus\x12.\n\x05phase\x18\x01 \x01(\x0e\x32\x18.grid.v1.ExperimentStateR\x05phase\x12\x43\n\x0fstart_timestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0estartTimestamp\x12\x45\n\x10\x66inish_timestamp\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0f\x66inishTimestamp\x12J\n\x0f\x65xecutor_status\x18\x04 \x01(\x0b\x32!.grid.v1.ExperimentExecutorStatusR\x0e\x65xecutorStatus\x12\x37\n\x06reason\x18\x05 \x01(\x0e\x32\x1f.grid.v1.ExperimentStatusReasonR\x06reason\x12\x18\n\x07message\x18\x06 \x01(\tR\x07message\x12G\n\x11\x64\x65leted_timestamp\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x10\x64\x65letedTimestamp\"\xe9\x01\n\x18\x45xperimentExecutorStatus\x12\x32\n\x08\x65xternal\x18\x01 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00R\x08\x65xternal\x12\x45\n\nkubernetes\x18\x02 \x01(\x0b\x32#.grid.v1.ExperimentKubernetesStatusH\x00R\nkubernetes\x12\x36\n\x05slurm\x18\x03 \x01(\x0b\x32\x1e.grid.v1.ExperimentSLURMStatusH\x00R\x05slurmB\x1a\n\x18\x65xecutor_specific_status\"2\n\x1a\x45xperimentKubernetesStatus\x12\x14\n\x05image\x18\x01 \x01(\tR\x05image\"\x17\n\x15\x45xperimentSLURMStatus*}\n\x12\x45xperimentStrategy\x12#\n\x1f\x45XPERIMENT_STRATEGY_UNSPECIFIED\x10\x00\x12\x1e\n\x1a\x45XPERIMENT_STRATEGY_SINGLE\x10\x01\x12\"\n\x1e\x45XPERIMENT_STRATEGY_MULTI_NODE\x10\x02*\xc9\x02\n\x0f\x45xperimentState\x12 \n\x1c\x45XPERIMENT_STATE_UNSPECIFIED\x10\x00\x12\x1c\n\x18\x45XPERIMENT_STATE_PENDING\x10\x01\x12\x1b\n\x17\x45XPERIMENT_STATE_QUEUED\x10\x02\x12\x1c\n\x18\x45XPERIMENT_STATE_RUNNING\x10\x03\x12\x1e\n\x1a\x45XPERIMENT_STATE_SUCCEEDED\x10\x04\x12\x1b\n\x17\x45XPERIMENT_STATE_FAILED\x10\x05\x12\x1e\n\x1a\x45XPERIMENT_STATE_CANCELLED\x10\x06\x12\x1c\n\x18\x45XPERIMENT_STATE_DELETED\x10\x07\x12\x1b\n\x17\x45XPERIMENT_STATE_PAUSED\x10\x08\x12#\n\x1f\x45XPERIMENT_STATE_IMAGE_BUILDING\x10\t*\x95\x04\n\x16\x45xperimentStatusReason\x12(\n$EXPERIMENT_STATUS_REASON_UNSPECIFIED\x10\x00\x12)\n%EXPERIMENT_STATUS_REASON_IMAGE_QUEUED\x10\x01\x12+\n\'EXPERIMENT_STATUS_REASON_IMAGE_BUILDING\x10\x02\x12,\n(EXPERIMENT_STATUS_REASON_IMAGE_COMPLETED\x10\x03\x12)\n%EXPERIMENT_STATUS_REASON_IMAGE_FAILED\x10\x04\x12>\n:EXPERIMENT_STATUS_REASON_RESUMABLE_EXPERIMENTS_UNSUPPORTED\x10\x05\x12+\n\'EXPERIMENT_STATUS_REASON_EXEC_USER_CODE\x10\x06\x12&\n\"EXPERIMENT_STATUS_REASON_EXEC_GRID\x10\x07\x12\x31\n-EXPERIMENT_STATUS_REASON_EXEC_POD_TERMINATION\x10\x08\x12.\n*EXPERIMENT_STATUS_REASON_POD_UNKNOWN_STATE\x10\t\x12(\n$EXPERIMENT_STATUS_REASON_POD_PENDING\x10\nB2Z0github.com/gridai/grid/grid-backend/apis/grid/v1b\x06proto3'
  ,
  dependencies=[grid_dot_v1_dot_metadata__pb2.DESCRIPTOR,grid_dot_v1_dot_datastore__pb2.DESCRIPTOR,grid_dot_v1_dot_image__pb2.DESCRIPTOR,grid_dot_v1_dot_resource__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])

_EXPERIMENTSTRATEGY = _descriptor.EnumDescriptor(
  name='ExperimentStrategy',
  full_name='grid.v1.ExperimentStrategy',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STRATEGY_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STRATEGY_SINGLE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STRATEGY_MULTI_NODE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2662,
  serialized_end=2787,
)
_sym_db.RegisterEnumDescriptor(_EXPERIMENTSTRATEGY)

ExperimentStrategy = enum_type_wrapper.EnumTypeWrapper(_EXPERIMENTSTRATEGY)
_EXPERIMENTSTATE = _descriptor.EnumDescriptor(
  name='ExperimentState',
  full_name='grid.v1.ExperimentState',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATE_PENDING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATE_QUEUED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATE_RUNNING', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATE_SUCCEEDED', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATE_FAILED', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATE_CANCELLED', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATE_DELETED', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATE_PAUSED', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATE_IMAGE_BUILDING', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2790,
  serialized_end=3119,
)
_sym_db.RegisterEnumDescriptor(_EXPERIMENTSTATE)

ExperimentState = enum_type_wrapper.EnumTypeWrapper(_EXPERIMENTSTATE)
_EXPERIMENTSTATUSREASON = _descriptor.EnumDescriptor(
  name='ExperimentStatusReason',
  full_name='grid.v1.ExperimentStatusReason',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATUS_REASON_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATUS_REASON_IMAGE_QUEUED', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATUS_REASON_IMAGE_BUILDING', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATUS_REASON_IMAGE_COMPLETED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATUS_REASON_IMAGE_FAILED', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATUS_REASON_RESUMABLE_EXPERIMENTS_UNSUPPORTED', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATUS_REASON_EXEC_USER_CODE', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATUS_REASON_EXEC_GRID', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATUS_REASON_EXEC_POD_TERMINATION', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATUS_REASON_POD_UNKNOWN_STATE', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPERIMENT_STATUS_REASON_POD_PENDING', index=10, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3122,
  serialized_end=3655,
)
_sym_db.RegisterEnumDescriptor(_EXPERIMENTSTATUSREASON)

ExperimentStatusReason = enum_type_wrapper.EnumTypeWrapper(_EXPERIMENTSTATUSREASON)
EXPERIMENT_STRATEGY_UNSPECIFIED = 0
EXPERIMENT_STRATEGY_SINGLE = 1
EXPERIMENT_STRATEGY_MULTI_NODE = 2
EXPERIMENT_STATE_UNSPECIFIED = 0
EXPERIMENT_STATE_PENDING = 1
EXPERIMENT_STATE_QUEUED = 2
EXPERIMENT_STATE_RUNNING = 3
EXPERIMENT_STATE_SUCCEEDED = 4
EXPERIMENT_STATE_FAILED = 5
EXPERIMENT_STATE_CANCELLED = 6
EXPERIMENT_STATE_DELETED = 7
EXPERIMENT_STATE_PAUSED = 8
EXPERIMENT_STATE_IMAGE_BUILDING = 9
EXPERIMENT_STATUS_REASON_UNSPECIFIED = 0
EXPERIMENT_STATUS_REASON_IMAGE_QUEUED = 1
EXPERIMENT_STATUS_REASON_IMAGE_BUILDING = 2
EXPERIMENT_STATUS_REASON_IMAGE_COMPLETED = 3
EXPERIMENT_STATUS_REASON_IMAGE_FAILED = 4
EXPERIMENT_STATUS_REASON_RESUMABLE_EXPERIMENTS_UNSUPPORTED = 5
EXPERIMENT_STATUS_REASON_EXEC_USER_CODE = 6
EXPERIMENT_STATUS_REASON_EXEC_GRID = 7
EXPERIMENT_STATUS_REASON_EXEC_POD_TERMINATION = 8
EXPERIMENT_STATUS_REASON_POD_UNKNOWN_STATE = 9
EXPERIMENT_STATUS_REASON_POD_PENDING = 10



_EXPERIMENT = _descriptor.Descriptor(
  name='Experiment',
  full_name='grid.v1.Experiment',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='metadata', full_name='grid.v1.Experiment.metadata', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='metadata', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='spec', full_name='grid.v1.Experiment.spec', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='spec', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='grid.v1.Experiment.status', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='status', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=192,
  serialized_end=347,
)


_SCRATCHSPACE = _descriptor.Descriptor(
  name='ScratchSpace',
  full_name='grid.v1.ScratchSpace',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='size_gb', full_name='grid.v1.ScratchSpace.size_gb', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sizeGb', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mount_path', full_name='grid.v1.ScratchSpace.mount_path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='mountPath', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=349,
  serialized_end=419,
)


_EXPERIMENTSPEC_ENVENTRY = _descriptor.Descriptor(
  name='EnvEntry',
  full_name='grid.v1.ExperimentSpec.EnvEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='grid.v1.ExperimentSpec.EnvEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='key', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='grid.v1.ExperimentSpec.EnvEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1108,
  serialized_end=1162,
)

_EXPERIMENTSPEC = _descriptor.Descriptor(
  name='ExperimentSpec',
  full_name='grid.v1.ExperimentSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='env', full_name='grid.v1.ExperimentSpec.env', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='env', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='executor', full_name='grid.v1.ExperimentSpec.executor', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='executor', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='desired_state', full_name='grid.v1.ExperimentSpec.desired_state', index=2,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='desiredState', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='run_id', full_name='grid.v1.ExperimentSpec.run_id', index=3,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='runId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_id', full_name='grid.v1.ExperimentSpec.user_id', index=4,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='userId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source_code', full_name='grid.v1.ExperimentSpec.source_code', index=5,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sourceCode', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='command', full_name='grid.v1.ExperimentSpec.command', index=6,
      number=9, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='command', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='relative_working_dir', full_name='grid.v1.ExperimentSpec.relative_working_dir', index=7,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relativeWorkingDir', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resources', full_name='grid.v1.ExperimentSpec.resources', index=8,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resources', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='inputs', full_name='grid.v1.ExperimentSpec.inputs', index=9,
      number=12, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='inputs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pause_automation', full_name='grid.v1.ExperimentSpec.pause_automation', index=10,
      number=13, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='pauseAutomation', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scratch', full_name='grid.v1.ExperimentSpec.scratch', index=11,
      number=14, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='scratch', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='framework', full_name='grid.v1.ExperimentSpec.framework', index=12,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='framework', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy', full_name='grid.v1.ExperimentSpec.strategy', index=13,
      number=16, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='strategy', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resumable', full_name='grid.v1.ExperimentSpec.resumable', index=14,
      number=17, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resumable', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_EXPERIMENTSPEC_ENVENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=422,
  serialized_end=1162,
)


_EXPERIMENTEXECUTOR = _descriptor.Descriptor(
  name='ExperimentExecutor',
  full_name='grid.v1.ExperimentExecutor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='cluster_id', full_name='grid.v1.ExperimentExecutor.cluster_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='clusterId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='external', full_name='grid.v1.ExperimentExecutor.external', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='external', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='kubernetes', full_name='grid.v1.ExperimentExecutor.kubernetes', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='kubernetes', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='slurm', full_name='grid.v1.ExperimentExecutor.slurm', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='slurm', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scheduler', full_name='grid.v1.ExperimentExecutor.scheduler', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='scheduler', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='executor', full_name='grid.v1.ExperimentExecutor.executor',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1165,
  serialized_end=1471,
)


_EXPERIMENTSCHEDULER = _descriptor.Descriptor(
  name='ExperimentScheduler',
  full_name='grid.v1.ExperimentScheduler',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='external', full_name='grid.v1.ExperimentScheduler.external', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='external', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='simple', full_name='grid.v1.ExperimentScheduler.simple', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='simple', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='scheduler', full_name='grid.v1.ExperimentScheduler.scheduler',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1474,
  serialized_end=1622,
)


_SIMPLEEXPERIMENTSCHEDULER = _descriptor.Descriptor(
  name='SimpleExperimentScheduler',
  full_name='grid.v1.SimpleExperimentScheduler',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='candidate_clusters', full_name='grid.v1.SimpleExperimentScheduler.candidate_clusters', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='candidateClusters', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1624,
  serialized_end=1698,
)


_EXPERIMENTSOURCECODE = _descriptor.Descriptor(
  name='ExperimentSourceCode',
  full_name='grid.v1.ExperimentSourceCode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='dsn', full_name='grid.v1.ExperimentSourceCode.dsn', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='dsn', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1700,
  serialized_end=1740,
)


_EXPERIMENTKUBERNETESEXECUTOR = _descriptor.Descriptor(
  name='ExperimentKubernetesExecutor',
  full_name='grid.v1.ExperimentKubernetesExecutor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance_type', full_name='grid.v1.ExperimentKubernetesExecutor.instance_type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='instanceType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='use_spot', full_name='grid.v1.ExperimentKubernetesExecutor.use_spot', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='useSpot', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='image', full_name='grid.v1.ExperimentKubernetesExecutor.image', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='image', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1743,
  serialized_end=1879,
)


_EXPERIMENTSLURMEXECUTOR = _descriptor.Descriptor(
  name='ExperimentSLURMExecutor',
  full_name='grid.v1.ExperimentSLURMExecutor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1881,
  serialized_end=1906,
)


_EXPERIMENTSTATUS = _descriptor.Descriptor(
  name='ExperimentStatus',
  full_name='grid.v1.ExperimentStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='phase', full_name='grid.v1.ExperimentStatus.phase', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='phase', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_timestamp', full_name='grid.v1.ExperimentStatus.start_timestamp', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='startTimestamp', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='finish_timestamp', full_name='grid.v1.ExperimentStatus.finish_timestamp', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='finishTimestamp', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='executor_status', full_name='grid.v1.ExperimentStatus.executor_status', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='executorStatus', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='reason', full_name='grid.v1.ExperimentStatus.reason', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='reason', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='grid.v1.ExperimentStatus.message', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='message', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deleted_timestamp', full_name='grid.v1.ExperimentStatus.deleted_timestamp', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='deletedTimestamp', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1909,
  serialized_end=2347,
)


_EXPERIMENTEXECUTORSTATUS = _descriptor.Descriptor(
  name='ExperimentExecutorStatus',
  full_name='grid.v1.ExperimentExecutorStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='external', full_name='grid.v1.ExperimentExecutorStatus.external', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='external', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='kubernetes', full_name='grid.v1.ExperimentExecutorStatus.kubernetes', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='kubernetes', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='slurm', full_name='grid.v1.ExperimentExecutorStatus.slurm', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='slurm', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='executor_specific_status', full_name='grid.v1.ExperimentExecutorStatus.executor_specific_status',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=2350,
  serialized_end=2583,
)


_EXPERIMENTKUBERNETESSTATUS = _descriptor.Descriptor(
  name='ExperimentKubernetesStatus',
  full_name='grid.v1.ExperimentKubernetesStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='image', full_name='grid.v1.ExperimentKubernetesStatus.image', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='image', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2585,
  serialized_end=2635,
)


_EXPERIMENTSLURMSTATUS = _descriptor.Descriptor(
  name='ExperimentSLURMStatus',
  full_name='grid.v1.ExperimentSLURMStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2637,
  serialized_end=2660,
)

_EXPERIMENT.fields_by_name['metadata'].message_type = grid_dot_v1_dot_metadata__pb2._METADATA
_EXPERIMENT.fields_by_name['spec'].message_type = _EXPERIMENTSPEC
_EXPERIMENT.fields_by_name['status'].message_type = _EXPERIMENTSTATUS
_EXPERIMENTSPEC_ENVENTRY.containing_type = _EXPERIMENTSPEC
_EXPERIMENTSPEC.fields_by_name['env'].message_type = _EXPERIMENTSPEC_ENVENTRY
_EXPERIMENTSPEC.fields_by_name['executor'].message_type = _EXPERIMENTEXECUTOR
_EXPERIMENTSPEC.fields_by_name['desired_state'].enum_type = _EXPERIMENTSTATE
_EXPERIMENTSPEC.fields_by_name['source_code'].message_type = _EXPERIMENTSOURCECODE
_EXPERIMENTSPEC.fields_by_name['resources'].message_type = grid_dot_v1_dot_resource__pb2._RESOURCES
_EXPERIMENTSPEC.fields_by_name['inputs'].message_type = grid_dot_v1_dot_datastore__pb2._DATASTOREINPUT
_EXPERIMENTSPEC.fields_by_name['scratch'].message_type = _SCRATCHSPACE
_EXPERIMENTSPEC.fields_by_name['strategy'].enum_type = _EXPERIMENTSTRATEGY
_EXPERIMENTEXECUTOR.fields_by_name['external'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_EXPERIMENTEXECUTOR.fields_by_name['kubernetes'].message_type = _EXPERIMENTKUBERNETESEXECUTOR
_EXPERIMENTEXECUTOR.fields_by_name['slurm'].message_type = _EXPERIMENTSLURMEXECUTOR
_EXPERIMENTEXECUTOR.fields_by_name['scheduler'].message_type = _EXPERIMENTSCHEDULER
_EXPERIMENTEXECUTOR.oneofs_by_name['executor'].fields.append(
  _EXPERIMENTEXECUTOR.fields_by_name['external'])
_EXPERIMENTEXECUTOR.fields_by_name['external'].containing_oneof = _EXPERIMENTEXECUTOR.oneofs_by_name['executor']
_EXPERIMENTEXECUTOR.oneofs_by_name['executor'].fields.append(
  _EXPERIMENTEXECUTOR.fields_by_name['kubernetes'])
_EXPERIMENTEXECUTOR.fields_by_name['kubernetes'].containing_oneof = _EXPERIMENTEXECUTOR.oneofs_by_name['executor']
_EXPERIMENTEXECUTOR.oneofs_by_name['executor'].fields.append(
  _EXPERIMENTEXECUTOR.fields_by_name['slurm'])
_EXPERIMENTEXECUTOR.fields_by_name['slurm'].containing_oneof = _EXPERIMENTEXECUTOR.oneofs_by_name['executor']
_EXPERIMENTSCHEDULER.fields_by_name['external'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_EXPERIMENTSCHEDULER.fields_by_name['simple'].message_type = _SIMPLEEXPERIMENTSCHEDULER
_EXPERIMENTSCHEDULER.oneofs_by_name['scheduler'].fields.append(
  _EXPERIMENTSCHEDULER.fields_by_name['external'])
_EXPERIMENTSCHEDULER.fields_by_name['external'].containing_oneof = _EXPERIMENTSCHEDULER.oneofs_by_name['scheduler']
_EXPERIMENTSCHEDULER.oneofs_by_name['scheduler'].fields.append(
  _EXPERIMENTSCHEDULER.fields_by_name['simple'])
_EXPERIMENTSCHEDULER.fields_by_name['simple'].containing_oneof = _EXPERIMENTSCHEDULER.oneofs_by_name['scheduler']
_EXPERIMENTKUBERNETESEXECUTOR.fields_by_name['image'].message_type = grid_dot_v1_dot_image__pb2._IMAGESPEC
_EXPERIMENTSTATUS.fields_by_name['phase'].enum_type = _EXPERIMENTSTATE
_EXPERIMENTSTATUS.fields_by_name['start_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_EXPERIMENTSTATUS.fields_by_name['finish_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_EXPERIMENTSTATUS.fields_by_name['executor_status'].message_type = _EXPERIMENTEXECUTORSTATUS
_EXPERIMENTSTATUS.fields_by_name['reason'].enum_type = _EXPERIMENTSTATUSREASON
_EXPERIMENTSTATUS.fields_by_name['deleted_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_EXPERIMENTEXECUTORSTATUS.fields_by_name['external'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_EXPERIMENTEXECUTORSTATUS.fields_by_name['kubernetes'].message_type = _EXPERIMENTKUBERNETESSTATUS
_EXPERIMENTEXECUTORSTATUS.fields_by_name['slurm'].message_type = _EXPERIMENTSLURMSTATUS
_EXPERIMENTEXECUTORSTATUS.oneofs_by_name['executor_specific_status'].fields.append(
  _EXPERIMENTEXECUTORSTATUS.fields_by_name['external'])
_EXPERIMENTEXECUTORSTATUS.fields_by_name['external'].containing_oneof = _EXPERIMENTEXECUTORSTATUS.oneofs_by_name['executor_specific_status']
_EXPERIMENTEXECUTORSTATUS.oneofs_by_name['executor_specific_status'].fields.append(
  _EXPERIMENTEXECUTORSTATUS.fields_by_name['kubernetes'])
_EXPERIMENTEXECUTORSTATUS.fields_by_name['kubernetes'].containing_oneof = _EXPERIMENTEXECUTORSTATUS.oneofs_by_name['executor_specific_status']
_EXPERIMENTEXECUTORSTATUS.oneofs_by_name['executor_specific_status'].fields.append(
  _EXPERIMENTEXECUTORSTATUS.fields_by_name['slurm'])
_EXPERIMENTEXECUTORSTATUS.fields_by_name['slurm'].containing_oneof = _EXPERIMENTEXECUTORSTATUS.oneofs_by_name['executor_specific_status']
DESCRIPTOR.message_types_by_name['Experiment'] = _EXPERIMENT
DESCRIPTOR.message_types_by_name['ScratchSpace'] = _SCRATCHSPACE
DESCRIPTOR.message_types_by_name['ExperimentSpec'] = _EXPERIMENTSPEC
DESCRIPTOR.message_types_by_name['ExperimentExecutor'] = _EXPERIMENTEXECUTOR
DESCRIPTOR.message_types_by_name['ExperimentScheduler'] = _EXPERIMENTSCHEDULER
DESCRIPTOR.message_types_by_name['SimpleExperimentScheduler'] = _SIMPLEEXPERIMENTSCHEDULER
DESCRIPTOR.message_types_by_name['ExperimentSourceCode'] = _EXPERIMENTSOURCECODE
DESCRIPTOR.message_types_by_name['ExperimentKubernetesExecutor'] = _EXPERIMENTKUBERNETESEXECUTOR
DESCRIPTOR.message_types_by_name['ExperimentSLURMExecutor'] = _EXPERIMENTSLURMEXECUTOR
DESCRIPTOR.message_types_by_name['ExperimentStatus'] = _EXPERIMENTSTATUS
DESCRIPTOR.message_types_by_name['ExperimentExecutorStatus'] = _EXPERIMENTEXECUTORSTATUS
DESCRIPTOR.message_types_by_name['ExperimentKubernetesStatus'] = _EXPERIMENTKUBERNETESSTATUS
DESCRIPTOR.message_types_by_name['ExperimentSLURMStatus'] = _EXPERIMENTSLURMSTATUS
DESCRIPTOR.enum_types_by_name['ExperimentStrategy'] = _EXPERIMENTSTRATEGY
DESCRIPTOR.enum_types_by_name['ExperimentState'] = _EXPERIMENTSTATE
DESCRIPTOR.enum_types_by_name['ExperimentStatusReason'] = _EXPERIMENTSTATUSREASON
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Experiment = _reflection.GeneratedProtocolMessageType('Experiment', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENT,
  '__module__' : 'grid.v1.experiment_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.Experiment)
  })
_sym_db.RegisterMessage(Experiment)

ScratchSpace = _reflection.GeneratedProtocolMessageType('ScratchSpace', (_message.Message,), {
  'DESCRIPTOR' : _SCRATCHSPACE,
  '__module__' : 'grid.v1.experiment_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ScratchSpace)
  })
_sym_db.RegisterMessage(ScratchSpace)

ExperimentSpec = _reflection.GeneratedProtocolMessageType('ExperimentSpec', (_message.Message,), {

  'EnvEntry' : _reflection.GeneratedProtocolMessageType('EnvEntry', (_message.Message,), {
    'DESCRIPTOR' : _EXPERIMENTSPEC_ENVENTRY,
    '__module__' : 'grid.v1.experiment_pb2'
    # @@protoc_insertion_point(class_scope:grid.v1.ExperimentSpec.EnvEntry)
    })
  ,
  'DESCRIPTOR' : _EXPERIMENTSPEC,
  '__module__' : 'grid.v1.experiment_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ExperimentSpec)
  })
_sym_db.RegisterMessage(ExperimentSpec)
_sym_db.RegisterMessage(ExperimentSpec.EnvEntry)

ExperimentExecutor = _reflection.GeneratedProtocolMessageType('ExperimentExecutor', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENTEXECUTOR,
  '__module__' : 'grid.v1.experiment_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ExperimentExecutor)
  })
_sym_db.RegisterMessage(ExperimentExecutor)

ExperimentScheduler = _reflection.GeneratedProtocolMessageType('ExperimentScheduler', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENTSCHEDULER,
  '__module__' : 'grid.v1.experiment_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ExperimentScheduler)
  })
_sym_db.RegisterMessage(ExperimentScheduler)

SimpleExperimentScheduler = _reflection.GeneratedProtocolMessageType('SimpleExperimentScheduler', (_message.Message,), {
  'DESCRIPTOR' : _SIMPLEEXPERIMENTSCHEDULER,
  '__module__' : 'grid.v1.experiment_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.SimpleExperimentScheduler)
  })
_sym_db.RegisterMessage(SimpleExperimentScheduler)

ExperimentSourceCode = _reflection.GeneratedProtocolMessageType('ExperimentSourceCode', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENTSOURCECODE,
  '__module__' : 'grid.v1.experiment_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ExperimentSourceCode)
  })
_sym_db.RegisterMessage(ExperimentSourceCode)

ExperimentKubernetesExecutor = _reflection.GeneratedProtocolMessageType('ExperimentKubernetesExecutor', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENTKUBERNETESEXECUTOR,
  '__module__' : 'grid.v1.experiment_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ExperimentKubernetesExecutor)
  })
_sym_db.RegisterMessage(ExperimentKubernetesExecutor)

ExperimentSLURMExecutor = _reflection.GeneratedProtocolMessageType('ExperimentSLURMExecutor', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENTSLURMEXECUTOR,
  '__module__' : 'grid.v1.experiment_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ExperimentSLURMExecutor)
  })
_sym_db.RegisterMessage(ExperimentSLURMExecutor)

ExperimentStatus = _reflection.GeneratedProtocolMessageType('ExperimentStatus', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENTSTATUS,
  '__module__' : 'grid.v1.experiment_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ExperimentStatus)
  })
_sym_db.RegisterMessage(ExperimentStatus)

ExperimentExecutorStatus = _reflection.GeneratedProtocolMessageType('ExperimentExecutorStatus', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENTEXECUTORSTATUS,
  '__module__' : 'grid.v1.experiment_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ExperimentExecutorStatus)
  })
_sym_db.RegisterMessage(ExperimentExecutorStatus)

ExperimentKubernetesStatus = _reflection.GeneratedProtocolMessageType('ExperimentKubernetesStatus', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENTKUBERNETESSTATUS,
  '__module__' : 'grid.v1.experiment_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ExperimentKubernetesStatus)
  })
_sym_db.RegisterMessage(ExperimentKubernetesStatus)

ExperimentSLURMStatus = _reflection.GeneratedProtocolMessageType('ExperimentSLURMStatus', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENTSLURMSTATUS,
  '__module__' : 'grid.v1.experiment_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ExperimentSLURMStatus)
  })
_sym_db.RegisterMessage(ExperimentSLURMStatus)


DESCRIPTOR._options = None
_EXPERIMENTSPEC_ENVENTRY._options = None
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from grid.protos.grid.v1 import experiment_service_pb2 as grid_dot_v1_dot_experiment__service__pb2


class ExperimentServiceStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.CreateExperiment = channel.unary_unary(
        '/grid.v1.ExperimentService/CreateExperiment',
        request_serializer=grid_dot_v1_dot_experiment__service__pb2.CreateExperimentRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_experiment__service__pb2.CreateExperimentResponse.FromString,
        )
    self.GetExperiment = channel.unary_unary(
        '/grid.v1.ExperimentService/GetExperiment',
        request_serializer=grid_dot_v1_dot_experiment__service__pb2.GetExperimentRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_experiment__service__pb2.GetExperimentResponse.FromString,
        )
    self.ListExperiments = channel.unary_unary(
        '/grid.v1.ExperimentService/ListExperiments',
        request_serializer=grid_dot_v1_dot_experiment__service__pb2.ListExperimentsRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_experiment__service__pb2.ListExperimentsResponse.FromString,
        )
    self.UpdateExperiment = channel.unary_unary(
        '/grid.v1.ExperimentService/UpdateExperiment',
        request_serializer=grid_dot_v1_dot_experiment__service__pb2.UpdateExperimentRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_experiment__service__pb2.UpdateExperimentResponse.FromString,
        )
    self.UpdateExperimentStatus = channel.unary_unary(
        '/grid.v1.ExperimentService/UpdateExperimentStatus',
        request_serializer=grid_dot_v1_dot_experiment__service__pb2.UpdateExperimentStatusRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_experiment__service__pb2.UpdateExperimentStatusResponse.FromString,
        )
    self.DeleteExperiment = channel.unary_unary(
        '/grid.v1.ExperimentService/DeleteExperiment',
        request_serializer=grid_dot_v1_dot_experiment__service__pb2.DeleteExperimentRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_experiment__service__pb2.DeleteExperimentResponse.FromString,
        )
    self.WatchExperiments = channel.unary_stream(
        '/grid.v1.ExperimentService/WatchExperiments',
        request_serializer=grid_dot_v1_dot_experiment__service__pb2.WatchExperimentsRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_experiment__service__pb2.WatchExperimentsResponse.FromString,
        )


class ExperimentServiceServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def CreateExperiment(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetExperiment(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListExperiments(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UpdateExperiment(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UpdateExperimentStatus(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DeleteExperiment(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def WatchExperiments(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_ExperimentServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'CreateExperiment': grpc.unary_unary_rpc_method_handler(
          servicer.CreateExperiment,
          request_deserializer=grid_dot_v1_dot_experiment__service__pb2.CreateExperimentRequest.FromString,
          response_serializer=grid_dot_v1_dot_experiment__service__pb2.CreateExperimentResponse.SerializeToString,
      ),
      'GetExperiment': grpc.unary_unary_rpc_method_handler(
          servicer.GetExperiment,
          request_deserializer=grid_dot_v1_dot_experiment__service__pb2.GetExperimentRequest.FromString,
          response_serializer=grid_dot_v1_dot_experiment__service__pb2.GetExperimentResponse.SerializeToString,
      ),
      'ListExperiments': grpc.unary_unary_rpc_method_handler(
          servicer.ListExperiments,
          request_deserializer=grid_dot_v1_dot_experiment__service__pb2.ListExperimentsRequest.FromString,
          response_serializer=grid_dot_v1_dot_experiment__service__pb2.ListExperimentsResponse.SerializeToString,
      ),
      'UpdateExperiment': grpc.unary_unary_rpc_method_handler(
          servicer.UpdateExperiment,
          request_deserializer=grid_dot_v1_dot_experiment__service__pb2.UpdateExperimentRequest.FromString,
          response_serializer=grid_dot_v1_dot_experiment__service__pb2.UpdateExperimentResponse.SerializeToString,
      ),
      'UpdateExperimentStatus': grpc.unary_unary_rpc_method_handler(
          servicer.UpdateExperimentStatus,
          request_deserializer=grid_dot_v1_dot_experiment__service__pb2.UpdateExperimentStatusRequest.FromString,
          response_serializer=grid_dot_v1_dot_experiment__service__pb2.UpdateExperimentStatusResponse.SerializeToString,
      ),
      'DeleteExperiment': grpc.unary_unary_rpc_method_handler(
          servicer.DeleteExperiment,
          request_deserializer=grid_dot_v1_dot_experiment__service__pb2.DeleteExperimentRequest.FromString,
          response_serializer=grid_dot_v1_dot_experiment__service__pb2.DeleteExperimentResponse.SerializeToString,
      ),
      'WatchExperiments': grpc.unary_stream_rpc_method_handler(
          servicer.WatchExperiments,
          request_deserializer=grid_dot_v1_dot_experiment__service__pb2.WatchExperimentsRequest.FromString,
          response_serializer=grid_dot_v1_dot_experiment__service__pb2.WatchExperimentsResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'grid.v1.ExperimentService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



import asyncio
from datetime import datetime
import os
from pathlib import Path
import time
from typing import List, Optional, Union
from urllib.parse import urlparse

import click

from grid.sdk._gql.queries import delete_datastore, get_available_datastores, get_user_teams
from grid.sdk.team import Team
from grid.sdk.user import SDKUser, User
from grid.sdk.utils.datastore_uploader import create_datastore_session

DATASTORE_TERMINAL_STATES = ['FAILED', 'SUCCEEDED']


def fail_if_deleted(func):
    """Decorator which raises an exception at access time if the datastore has been deleted.
    """
    def wrapper(self, *args, **kwargs):
        if not self._alive:
            raise RuntimeError('Cannot perform operation on deleted datastore') from None
        return func(self, *args, **kwargs)

    return wrapper


def fail_if_datastore_currently_exists(func):
    """Decorator which raises an exception at access time if the datastore has been uploaded to Grid.
    """
    def wrapper(self, *args, **kwargs):
        if self._currently_exists:
            raise RuntimeError('Property only can be accessed before the datastore is uploaded to Grid.') from None
        return func(self, *args, **kwargs)

    return wrapper


def fail_if_datastore_does_not_exist(func):
    """Decorator which raises an exception at access time if the datastore has not been uploaded to Grid.
    """
    def wrapper(self, *args, **kwargs):
        if not self._currently_exists:
            raise RuntimeError('Property only can be accessed before the datastore is uploaded to Grid.') from None
        return func(self, *args, **kwargs)

    return wrapper


def list_all_available_datastores(include_teams: bool = False) -> List['DataStore']:

    datastores = []
    user_dstores_data = get_available_datastores()
    for dstore_data in user_dstores_data:
        user = User(
            username=dstore_data['userDetails']['username'],
            user_id=dstore_data['userDetails']['id'],
            first_name=dstore_data['userDetails']['firstName'],
            last_name=dstore_data['userDetails']['lastName'],
        )
        dstore = Datastore._from_existing(
            name=dstore_data['name'],
            version=int(dstore_data['version']),
            owner=user,
            team=None,
            size=dstore_data['size'],
            created_at=datetime.fromisoformat(dstore_data['createdAt']),
            snapshot_status=dstore_data['snapshotStatus'],
            datastore_id=dstore_data['id'],
            cluster_id=dstore_data['clusterId'],
        )
        datastores.append(dstore)

    # If ``include_teams`` is set, add datastores registered to the team.

    teams = []
    if include_teams is True:
        teams_data = get_user_teams()
        for team_data in teams_data:
            members = {}
            for member_data in team_data['members']:
                user = User(
                    username=member_data['username'],
                    user_id=member_data['id'],
                    first_name=member_data['firstName'],
                    last_name=member_data['lastName'],
                )
                members[user.username] = user

            team = Team(
                team_id=team_data['id'],
                name=team_data['name'],
                created_at=team_data['createdAt'],
                role=team_data['role'],
                members=members,
            )
            teams.append(team)

    for team in teams:
        team_dstores_data = get_available_datastores(team_id=team.team_id)
        for dstore_data in team_dstores_data:
            user = User(
                user_id=dstore_data['userDetails']['id'],
                username=dstore_data['userDetails']['username'],
                first_name=dstore_data['userDetails']['firstName'],
                last_name=dstore_data['userDetails']['lastName']
            )

            dstore = Datastore._from_existing(
                name=dstore_data['name'],
                version=int(dstore_data['version']),
                owner=user,
                team=team,
                size=dstore_data['size'],
                created_at=datetime.fromisoformat(dstore_data['createdAt']),
                snapshot_status=dstore_data['snapshotStatus'],
                datastore_id=dstore_data['id'],
                cluster_id=dstore_data['clusterId'],
            )
            datastores.append(dstore)

    return datastores


class Datastore:
    _name: str
    _datastore_id: str
    _version: int
    _source: Optional[Union[str, Path]] = None
    _compression: bool
    _snapshot_status: str
    _created_at: datetime
    _owner: User
    _team: Optional[Team]
    _cluster_id: str
    _size: str

    _alive: bool = True
    _currently_exists: bool = False

    def __init__(
        self,
        name: Optional[str] = None,
        source: Optional[os.PathLike] = None,
        version: int = 1,
        compression: Optional[bool] = None,
        owner: Optional[str] = None,
        team: Optional[str] = None,
        cluster_id: Optional[str] = None,
    ):
        """Initialize a new DataStore Object.

        If a DataStore with the given name, version, team and cluster already exists,
        then the object returned will be able to interact with the existing DataStore.

        Alternatively, if the DataStore is going to be created for the first time, then
        the ``source`` and ``compression`` parameters can be used to specify the location
        of the DataStore on disk (or at a remote location). and to optionally compress the
        data before uploading.

        After initializing the datastore object, the data itself can be uploaded by calling
        the ``upload()`` method.

        Parameters
        ----------
        name
            The name of the DataStore.
        version
            The version of the DataStore.
        source
            The location of the DataStore on disk or at a remote location.
        compression
            True if the DataStore should be compressed before uploading. Otherwise False.
        owner
            The username of the owner of the DataStore.
        team
            The name of the team that owns the DataStore.
        cluster_id
            The name of the cluster that the DataStore should be uploaded to.
        """
        if self._currently_exists is True:
            return

        if owner is None:
            _sdk_user = SDKUser.from_logged_in_user()
            owner = User(
                user_id=_sdk_user.user_id,
                username=_sdk_user.username,
                first_name=_sdk_user.first_name,
                last_name=_sdk_user.last_name
            )

        if name is None:
            name = parse_name_from_source(source)

        self._name = name
        self._version = version
        self._source = source
        self._compression = compression
        self._owner = owner
        self._team = team
        self._cluster_id = cluster_id

        self._datastore_id = None
        self._snapshot_status = None
        self._created_at = None
        self._size = None

        for datastore in list_all_available_datastores(True):
            if datastore == self:
                self._name = datastore.name
                self._datastore_id = datastore.datastore_id
                self._version = datastore._version
                self._source = datastore._source
                self._compression = datastore.compression
                self._snapshot_status = datastore.snapshot_status
                self._created_at = datastore.created_at
                self._owner = datastore.owner
                self._team = datastore.team
                self._cluster_id = datastore.cluster_id
                self._size = datastore.size

                self._alive = True
                self._currently_exists = True

    @classmethod
    def _from_existing(
        cls,
        name: str,
        version: int,
        owner: User,
        size: str,
        created_at: datetime,
        snapshot_status: str,
        datastore_id: str,
        cluster_id: str,
        team: Optional[Team] = None,
    ):
        cls._currently_exists = True
        try:
            dstore = cls()
        finally:
            # need to reset value so that other instances can be created in the future.
            cls._currently_exists = False

        dstore._name = name
        dstore._version = version
        dstore._datastore_id = datastore_id
        dstore._size = size
        dstore._created_at = created_at
        dstore._snapshot_status = snapshot_status
        dstore._owner = owner
        dstore._team = team
        dstore._cluster_id = cluster_id

        dstore._source = None
        dstore._compression = None

        dstore._currently_exists = True

        return dstore

    # ------------------ Attributes Only Valid Before Upload ---------------

    @property
    @fail_if_deleted
    def source(self) -> Union[str, Path]:
        return self._source

    @property
    @fail_if_deleted
    def compression(self) -> bool:
        return self._compression

    # ------------------ Attributes Fully Active After Upload ---------------

    @property
    @fail_if_deleted
    def name(self) -> str:
        return self._name

    @property
    @fail_if_deleted
    def version(self) -> int:
        return self._version

    @property
    @fail_if_deleted
    def owner(self) -> User:
        return self._owner

    @property
    @fail_if_deleted
    def team(self) -> Team:
        return self._team

    @property
    @fail_if_deleted
    def created_at(self) -> datetime:
        return self._created_at

    @property
    @fail_if_deleted
    def size(self) -> str:
        return self._size

    @property
    @fail_if_deleted
    def snapshot_status(self) -> str:
        if (self._snapshot_status and
            (self._snapshot_status.upper() not in DATASTORE_TERMINAL_STATES)) or ((self._snapshot_status is None) and
                                                                                  (self._currently_exists is True)):
            self._update_status()
        return self._snapshot_status

    @property
    @fail_if_deleted
    def cluster_id(self) -> str:
        return self._cluster_id

    @property
    @fail_if_deleted
    def datastore_id(self) -> str:
        return self._datastore_id

    # -------------- Dunder Methods ----------------------

    @fail_if_deleted
    def __repr__(self):
        if self._currently_exists:
            res = (
                f"{self.__class__.__name__}(\n"
                f"  name={self.name}, \n"
                f"  version={self.version}, \n"
                f"  size={self.size}, \n"
                f"  created_at={self.created_at}, \n"
                f"  owner={self.owner}, \n"
                f"  team={self.team}, \n"
                f"  cluster_id={self.cluster_id}, \n"
                f"  datastore_id={self.datastore_id}\n"
                f"  )"
            )
        else:
            res = (
                f"{self.__class__.__name__}(\n"
                f"  name={self.name}, \n"
                f"  version={self.version}, \n"
                f"  source={self.source}, \n"
                f"  compression={self.compression} \n"
                f"  owner={self.owner}, \n"
                f"  team={self.team}, \n"
                f"  cluster_id={self.cluster_id}, \n"
                f")"
            )

        return res

    @fail_if_deleted
    def __str__(self):
        return repr(self)

    @fail_if_deleted
    def __eq__(self, other: 'Datastore'):
        # need to handle case where attributes of a DataStore are not `User` or `Team`
        # classes. This is the case before the datastore is uploaded.
        self_team = self._team.team_id if hasattr(self._team, 'team_id') else self._team
        other_team = other._team.team_id if hasattr(other._team, 'team_id') else self._team

        self_owner = self._owner.user_id if hasattr(self._owner, 'user_id') else self._owner
        other_owner = other._owner.user_id if hasattr(other._owner, 'user_id') else other.owner

        return (
            self.__class__.__qualname__ == other.__class__.__qualname__ and self._name == other._name
            and self._version == other._version and self_owner == other_owner and self_team == other_team
        )

    @fail_if_deleted
    def __hash__(self):
        return hash((
            self._name, self._datastore_id, self._version, self._size, self._created_at, self._snapshot_status,
            self._owner, self._team, self._source, self._compression, self._cluster_id, self._alive,
            self._currently_exists
        ))

    # ---------------------  Internal Methods ----------------------

    @fail_if_deleted
    def _update_status(self):
        """Refreshes the``snapshot_status`` attribute by querying the Grid API.
        """
        tid = None if self._team is None else self.team.team_id
        all_dstore_data = get_available_datastores(team_id=tid)

        for dstore_data in all_dstore_data:
            if dstore_data['id'] == self._datastore_id:
                self._snapshot_status = dstore_data['snapshotStatus'] or "unknown"

    # -------------------  Public Facing Methods ----------------------

    @fail_if_deleted
    @fail_if_datastore_does_not_exist
    def delete(self):
        """Deletes the datastore from the grid system
        """
        delete_datastore(name=self.name, version=self.version, cluster=self.cluster_id)
        self._alive = False

    @fail_if_deleted
    @fail_if_datastore_currently_exists
    def upload(self):
        session = create_datastore_session(
            name=self.name,
            source=self.source,
            compression=self.compression,
            cluster=self.cluster_id,
        )
        session.upload()

        # Since we can't query just a single datastore by it's ID (THIS IS INSANE!)
        # we just grab the entire world of datastores and check to see which one
        # has the same attributes as this (as determined by this classes __eq__ method)

        time.sleep(1)  # give the backend time to process the new record.
        for datastore in list_all_available_datastores(include_teams=True):
            if datastore == self:
                self._name = datastore._name
                self._datastore_id = datastore._datastore_id
                self._version = datastore._version
                self._source = datastore._source
                self._compression = datastore._compression
                self._snapshot_status = datastore._snapshot_status
                self._created_at = datastore._created_at
                self._owner = datastore._owner
                self._team = datastore._team
                self._cluster_id = datastore._cluster_id
                self._size = datastore._size

                self._alive = True
                self._currently_exists = True


def parse_name_from_source(ctx, _, name: Optional[str] = None) -> str:
    """Parses datastore name from source if name isn't provided"""
    if name:
        return name

    source = ctx.params['source']
    try:
        parse_result = urlparse(source)
    except ValueError:
        raise click.ClickException("Invalid source for datastore, please input only a local filepath or valid url")

    path = Path(parse_result.path)
    base = path.name.split(".")[0]
    return base.lower().strip()



# coding: utf-8

"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from grid.openapi.configuration import Configuration


class V1ClusterStatus(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'driver_status': 'V1ClusterDriverStatus',
        'gridlet_version': 'str',
        'last_healthcheck_timestamp': 'datetime',
        'phase': 'V1ClusterState',
        'reason': 'str'
    }

    attribute_map = {
        'driver_status': 'driverStatus',
        'gridlet_version': 'gridletVersion',
        'last_healthcheck_timestamp': 'lastHealthcheckTimestamp',
        'phase': 'phase',
        'reason': 'reason'
    }

    def __init__(self, driver_status=None, gridlet_version=None, last_healthcheck_timestamp=None, phase=None, reason=None, _configuration=None):  # noqa: E501
        """V1ClusterStatus - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._driver_status = None
        self._gridlet_version = None
        self._last_healthcheck_timestamp = None
        self._phase = None
        self._reason = None
        self.discriminator = None

        if driver_status is not None:
            self.driver_status = driver_status
        if gridlet_version is not None:
            self.gridlet_version = gridlet_version
        if last_healthcheck_timestamp is not None:
            self.last_healthcheck_timestamp = last_healthcheck_timestamp
        if phase is not None:
            self.phase = phase
        if reason is not None:
            self.reason = reason

    @property
    def driver_status(self):
        """Gets the driver_status of this V1ClusterStatus.  # noqa: E501


        :return: The driver_status of this V1ClusterStatus.  # noqa: E501
        :rtype: V1ClusterDriverStatus
        """
        return self._driver_status

    @driver_status.setter
    def driver_status(self, driver_status):
        """Sets the driver_status of this V1ClusterStatus.


        :param driver_status: The driver_status of this V1ClusterStatus.  # noqa: E501
        :type: V1ClusterDriverStatus
        """

        self._driver_status = driver_status

    @property
    def gridlet_version(self):
        """Gets the gridlet_version of this V1ClusterStatus.  # noqa: E501


        :return: The gridlet_version of this V1ClusterStatus.  # noqa: E501
        :rtype: str
        """
        return self._gridlet_version

    @gridlet_version.setter
    def gridlet_version(self, gridlet_version):
        """Sets the gridlet_version of this V1ClusterStatus.


        :param gridlet_version: The gridlet_version of this V1ClusterStatus.  # noqa: E501
        :type: str
        """

        self._gridlet_version = gridlet_version

    @property
    def last_healthcheck_timestamp(self):
        """Gets the last_healthcheck_timestamp of this V1ClusterStatus.  # noqa: E501


        :return: The last_healthcheck_timestamp of this V1ClusterStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._last_healthcheck_timestamp

    @last_healthcheck_timestamp.setter
    def last_healthcheck_timestamp(self, last_healthcheck_timestamp):
        """Sets the last_healthcheck_timestamp of this V1ClusterStatus.


        :param last_healthcheck_timestamp: The last_healthcheck_timestamp of this V1ClusterStatus.  # noqa: E501
        :type: datetime
        """

        self._last_healthcheck_timestamp = last_healthcheck_timestamp

    @property
    def phase(self):
        """Gets the phase of this V1ClusterStatus.  # noqa: E501


        :return: The phase of this V1ClusterStatus.  # noqa: E501
        :rtype: V1ClusterState
        """
        return self._phase

    @phase.setter
    def phase(self, phase):
        """Sets the phase of this V1ClusterStatus.


        :param phase: The phase of this V1ClusterStatus.  # noqa: E501
        :type: V1ClusterState
        """

        self._phase = phase

    @property
    def reason(self):
        """Gets the reason of this V1ClusterStatus.  # noqa: E501


        :return: The reason of this V1ClusterStatus.  # noqa: E501
        :rtype: str
        """
        return self._reason

    @reason.setter
    def reason(self, reason):
        """Sets the reason of this V1ClusterStatus.


        :param reason: The reason of this V1ClusterStatus.  # noqa: E501
        :type: str
        """

        self._reason = reason

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1ClusterStatus, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1ClusterStatus):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1ClusterStatus):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from grid.openapi.configuration import Configuration


class V1RunSpec(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'actions': 'V1RunActions',
        'cluster_id': 'str',
        'datastores': 'list[V1DatastoreMounts]',
        'desired_state': 'V1RunState',
        'dry_run': 'bool',
        'env': 'dict(str, str)',
        'image': 'Externalv1ImageSpec',
        'instance_type': 'str',
        'nodes': 'int',
        'payload': 'str',
        'resources': 'V1Resources',
        'run_controller_id': 'str',
        'scratch': 'list[Externalv1ScratchSpace]',
        'source_code': 'str',
        'sweep_options': 'dict(str, str)',
        'sweep_type': 'str',
        'use_spot': 'bool',
        'user_id': 'str'
    }

    attribute_map = {
        'actions': 'actions',
        'cluster_id': 'clusterId',
        'datastores': 'datastores',
        'desired_state': 'desiredState',
        'dry_run': 'dryRun',
        'env': 'env',
        'image': 'image',
        'instance_type': 'instanceType',
        'nodes': 'nodes',
        'payload': 'payload',
        'resources': 'resources',
        'run_controller_id': 'runControllerId',
        'scratch': 'scratch',
        'source_code': 'sourceCode',
        'sweep_options': 'sweepOptions',
        'sweep_type': 'sweepType',
        'use_spot': 'useSpot',
        'user_id': 'userId'
    }

    def __init__(self, actions=None, cluster_id=None, datastores=None, desired_state=None, dry_run=None, env=None, image=None, instance_type=None, nodes=None, payload=None, resources=None, run_controller_id=None, scratch=None, source_code=None, sweep_options=None, sweep_type=None, use_spot=None, user_id=None, _configuration=None):  # noqa: E501
        """V1RunSpec - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._actions = None
        self._cluster_id = None
        self._datastores = None
        self._desired_state = None
        self._dry_run = None
        self._env = None
        self._image = None
        self._instance_type = None
        self._nodes = None
        self._payload = None
        self._resources = None
        self._run_controller_id = None
        self._scratch = None
        self._source_code = None
        self._sweep_options = None
        self._sweep_type = None
        self._use_spot = None
        self._user_id = None
        self.discriminator = None

        if actions is not None:
            self.actions = actions
        if cluster_id is not None:
            self.cluster_id = cluster_id
        if datastores is not None:
            self.datastores = datastores
        if desired_state is not None:
            self.desired_state = desired_state
        if dry_run is not None:
            self.dry_run = dry_run
        if env is not None:
            self.env = env
        if image is not None:
            self.image = image
        if instance_type is not None:
            self.instance_type = instance_type
        if nodes is not None:
            self.nodes = nodes
        if payload is not None:
            self.payload = payload
        if resources is not None:
            self.resources = resources
        if run_controller_id is not None:
            self.run_controller_id = run_controller_id
        if scratch is not None:
            self.scratch = scratch
        if source_code is not None:
            self.source_code = source_code
        if sweep_options is not None:
            self.sweep_options = sweep_options
        if sweep_type is not None:
            self.sweep_type = sweep_type
        if use_spot is not None:
            self.use_spot = use_spot
        if user_id is not None:
            self.user_id = user_id

    @property
    def actions(self):
        """Gets the actions of this V1RunSpec.  # noqa: E501


        :return: The actions of this V1RunSpec.  # noqa: E501
        :rtype: V1RunActions
        """
        return self._actions

    @actions.setter
    def actions(self, actions):
        """Sets the actions of this V1RunSpec.


        :param actions: The actions of this V1RunSpec.  # noqa: E501
        :type: V1RunActions
        """

        self._actions = actions

    @property
    def cluster_id(self):
        """Gets the cluster_id of this V1RunSpec.  # noqa: E501


        :return: The cluster_id of this V1RunSpec.  # noqa: E501
        :rtype: str
        """
        return self._cluster_id

    @cluster_id.setter
    def cluster_id(self, cluster_id):
        """Sets the cluster_id of this V1RunSpec.


        :param cluster_id: The cluster_id of this V1RunSpec.  # noqa: E501
        :type: str
        """

        self._cluster_id = cluster_id

    @property
    def datastores(self):
        """Gets the datastores of this V1RunSpec.  # noqa: E501


        :return: The datastores of this V1RunSpec.  # noqa: E501
        :rtype: list[V1DatastoreMounts]
        """
        return self._datastores

    @datastores.setter
    def datastores(self, datastores):
        """Sets the datastores of this V1RunSpec.


        :param datastores: The datastores of this V1RunSpec.  # noqa: E501
        :type: list[V1DatastoreMounts]
        """

        self._datastores = datastores

    @property
    def desired_state(self):
        """Gets the desired_state of this V1RunSpec.  # noqa: E501


        :return: The desired_state of this V1RunSpec.  # noqa: E501
        :rtype: V1RunState
        """
        return self._desired_state

    @desired_state.setter
    def desired_state(self, desired_state):
        """Sets the desired_state of this V1RunSpec.


        :param desired_state: The desired_state of this V1RunSpec.  # noqa: E501
        :type: V1RunState
        """

        self._desired_state = desired_state

    @property
    def dry_run(self):
        """Gets the dry_run of this V1RunSpec.  # noqa: E501


        :return: The dry_run of this V1RunSpec.  # noqa: E501
        :rtype: bool
        """
        return self._dry_run

    @dry_run.setter
    def dry_run(self, dry_run):
        """Sets the dry_run of this V1RunSpec.


        :param dry_run: The dry_run of this V1RunSpec.  # noqa: E501
        :type: bool
        """

        self._dry_run = dry_run

    @property
    def env(self):
        """Gets the env of this V1RunSpec.  # noqa: E501


        :return: The env of this V1RunSpec.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._env

    @env.setter
    def env(self, env):
        """Sets the env of this V1RunSpec.


        :param env: The env of this V1RunSpec.  # noqa: E501
        :type: dict(str, str)
        """

        self._env = env

    @property
    def image(self):
        """Gets the image of this V1RunSpec.  # noqa: E501


        :return: The image of this V1RunSpec.  # noqa: E501
        :rtype: Externalv1ImageSpec
        """
        return self._image

    @image.setter
    def image(self, image):
        """Sets the image of this V1RunSpec.


        :param image: The image of this V1RunSpec.  # noqa: E501
        :type: Externalv1ImageSpec
        """

        self._image = image

    @property
    def instance_type(self):
        """Gets the instance_type of this V1RunSpec.  # noqa: E501


        :return: The instance_type of this V1RunSpec.  # noqa: E501
        :rtype: str
        """
        return self._instance_type

    @instance_type.setter
    def instance_type(self, instance_type):
        """Sets the instance_type of this V1RunSpec.


        :param instance_type: The instance_type of this V1RunSpec.  # noqa: E501
        :type: str
        """

        self._instance_type = instance_type

    @property
    def nodes(self):
        """Gets the nodes of this V1RunSpec.  # noqa: E501


        :return: The nodes of this V1RunSpec.  # noqa: E501
        :rtype: int
        """
        return self._nodes

    @nodes.setter
    def nodes(self, nodes):
        """Sets the nodes of this V1RunSpec.


        :param nodes: The nodes of this V1RunSpec.  # noqa: E501
        :type: int
        """

        self._nodes = nodes

    @property
    def payload(self):
        """Gets the payload of this V1RunSpec.  # noqa: E501


        :return: The payload of this V1RunSpec.  # noqa: E501
        :rtype: str
        """
        return self._payload

    @payload.setter
    def payload(self, payload):
        """Sets the payload of this V1RunSpec.


        :param payload: The payload of this V1RunSpec.  # noqa: E501
        :type: str
        """

        self._payload = payload

    @property
    def resources(self):
        """Gets the resources of this V1RunSpec.  # noqa: E501


        :return: The resources of this V1RunSpec.  # noqa: E501
        :rtype: V1Resources
        """
        return self._resources

    @resources.setter
    def resources(self, resources):
        """Sets the resources of this V1RunSpec.


        :param resources: The resources of this V1RunSpec.  # noqa: E501
        :type: V1Resources
        """

        self._resources = resources

    @property
    def run_controller_id(self):
        """Gets the run_controller_id of this V1RunSpec.  # noqa: E501


        :return: The run_controller_id of this V1RunSpec.  # noqa: E501
        :rtype: str
        """
        return self._run_controller_id

    @run_controller_id.setter
    def run_controller_id(self, run_controller_id):
        """Sets the run_controller_id of this V1RunSpec.


        :param run_controller_id: The run_controller_id of this V1RunSpec.  # noqa: E501
        :type: str
        """

        self._run_controller_id = run_controller_id

    @property
    def scratch(self):
        """Gets the scratch of this V1RunSpec.  # noqa: E501


        :return: The scratch of this V1RunSpec.  # noqa: E501
        :rtype: list[Externalv1ScratchSpace]
        """
        return self._scratch

    @scratch.setter
    def scratch(self, scratch):
        """Sets the scratch of this V1RunSpec.


        :param scratch: The scratch of this V1RunSpec.  # noqa: E501
        :type: list[Externalv1ScratchSpace]
        """

        self._scratch = scratch

    @property
    def source_code(self):
        """Gets the source_code of this V1RunSpec.  # noqa: E501


        :return: The source_code of this V1RunSpec.  # noqa: E501
        :rtype: str
        """
        return self._source_code

    @source_code.setter
    def source_code(self, source_code):
        """Sets the source_code of this V1RunSpec.


        :param source_code: The source_code of this V1RunSpec.  # noqa: E501
        :type: str
        """

        self._source_code = source_code

    @property
    def sweep_options(self):
        """Gets the sweep_options of this V1RunSpec.  # noqa: E501


        :return: The sweep_options of this V1RunSpec.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._sweep_options

    @sweep_options.setter
    def sweep_options(self, sweep_options):
        """Sets the sweep_options of this V1RunSpec.


        :param sweep_options: The sweep_options of this V1RunSpec.  # noqa: E501
        :type: dict(str, str)
        """

        self._sweep_options = sweep_options

    @property
    def sweep_type(self):
        """Gets the sweep_type of this V1RunSpec.  # noqa: E501


        :return: The sweep_type of this V1RunSpec.  # noqa: E501
        :rtype: str
        """
        return self._sweep_type

    @sweep_type.setter
    def sweep_type(self, sweep_type):
        """Sets the sweep_type of this V1RunSpec.


        :param sweep_type: The sweep_type of this V1RunSpec.  # noqa: E501
        :type: str
        """

        self._sweep_type = sweep_type

    @property
    def use_spot(self):
        """Gets the use_spot of this V1RunSpec.  # noqa: E501


        :return: The use_spot of this V1RunSpec.  # noqa: E501
        :rtype: bool
        """
        return self._use_spot

    @use_spot.setter
    def use_spot(self, use_spot):
        """Sets the use_spot of this V1RunSpec.


        :param use_spot: The use_spot of this V1RunSpec.  # noqa: E501
        :type: bool
        """

        self._use_spot = use_spot

    @property
    def user_id(self):
        """Gets the user_id of this V1RunSpec.  # noqa: E501


        :return: The user_id of this V1RunSpec.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this V1RunSpec.


        :param user_id: The user_id of this V1RunSpec.  # noqa: E501
        :type: str
        """

        self._user_id = user_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1RunSpec, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1RunSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1RunSpec):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from grid.openapi.configuration import Configuration


class V1SessionSpec(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'cluster_id': 'str',
        'datastores': 'list[V1DatastoreInput]',
        'desired_state': 'V1SessionState',
        'instance_type': 'str',
        'pause_automation': 'bool',
        'resources': 'V1Resources',
        'use_spot': 'bool',
        'user_id': 'str'
    }

    attribute_map = {
        'cluster_id': 'clusterId',
        'datastores': 'datastores',
        'desired_state': 'desiredState',
        'instance_type': 'instanceType',
        'pause_automation': 'pauseAutomation',
        'resources': 'resources',
        'use_spot': 'useSpot',
        'user_id': 'userId'
    }

    def __init__(self, cluster_id=None, datastores=None, desired_state=None, instance_type=None, pause_automation=None, resources=None, use_spot=None, user_id=None, _configuration=None):  # noqa: E501
        """V1SessionSpec - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._cluster_id = None
        self._datastores = None
        self._desired_state = None
        self._instance_type = None
        self._pause_automation = None
        self._resources = None
        self._use_spot = None
        self._user_id = None
        self.discriminator = None

        if cluster_id is not None:
            self.cluster_id = cluster_id
        if datastores is not None:
            self.datastores = datastores
        if desired_state is not None:
            self.desired_state = desired_state
        if instance_type is not None:
            self.instance_type = instance_type
        if pause_automation is not None:
            self.pause_automation = pause_automation
        if resources is not None:
            self.resources = resources
        if use_spot is not None:
            self.use_spot = use_spot
        if user_id is not None:
            self.user_id = user_id

    @property
    def cluster_id(self):
        """Gets the cluster_id of this V1SessionSpec.  # noqa: E501


        :return: The cluster_id of this V1SessionSpec.  # noqa: E501
        :rtype: str
        """
        return self._cluster_id

    @cluster_id.setter
    def cluster_id(self, cluster_id):
        """Sets the cluster_id of this V1SessionSpec.


        :param cluster_id: The cluster_id of this V1SessionSpec.  # noqa: E501
        :type: str
        """

        self._cluster_id = cluster_id

    @property
    def datastores(self):
        """Gets the datastores of this V1SessionSpec.  # noqa: E501


        :return: The datastores of this V1SessionSpec.  # noqa: E501
        :rtype: list[V1DatastoreInput]
        """
        return self._datastores

    @datastores.setter
    def datastores(self, datastores):
        """Sets the datastores of this V1SessionSpec.


        :param datastores: The datastores of this V1SessionSpec.  # noqa: E501
        :type: list[V1DatastoreInput]
        """

        self._datastores = datastores

    @property
    def desired_state(self):
        """Gets the desired_state of this V1SessionSpec.  # noqa: E501


        :return: The desired_state of this V1SessionSpec.  # noqa: E501
        :rtype: V1SessionState
        """
        return self._desired_state

    @desired_state.setter
    def desired_state(self, desired_state):
        """Sets the desired_state of this V1SessionSpec.


        :param desired_state: The desired_state of this V1SessionSpec.  # noqa: E501
        :type: V1SessionState
        """

        self._desired_state = desired_state

    @property
    def instance_type(self):
        """Gets the instance_type of this V1SessionSpec.  # noqa: E501


        :return: The instance_type of this V1SessionSpec.  # noqa: E501
        :rtype: str
        """
        return self._instance_type

    @instance_type.setter
    def instance_type(self, instance_type):
        """Sets the instance_type of this V1SessionSpec.


        :param instance_type: The instance_type of this V1SessionSpec.  # noqa: E501
        :type: str
        """

        self._instance_type = instance_type

    @property
    def pause_automation(self):
        """Gets the pause_automation of this V1SessionSpec.  # noqa: E501


        :return: The pause_automation of this V1SessionSpec.  # noqa: E501
        :rtype: bool
        """
        return self._pause_automation

    @pause_automation.setter
    def pause_automation(self, pause_automation):
        """Sets the pause_automation of this V1SessionSpec.


        :param pause_automation: The pause_automation of this V1SessionSpec.  # noqa: E501
        :type: bool
        """

        self._pause_automation = pause_automation

    @property
    def resources(self):
        """Gets the resources of this V1SessionSpec.  # noqa: E501


        :return: The resources of this V1SessionSpec.  # noqa: E501
        :rtype: V1Resources
        """
        return self._resources

    @resources.setter
    def resources(self, resources):
        """Sets the resources of this V1SessionSpec.


        :param resources: The resources of this V1SessionSpec.  # noqa: E501
        :type: V1Resources
        """

        self._resources = resources

    @property
    def use_spot(self):
        """Gets the use_spot of this V1SessionSpec.  # noqa: E501


        :return: The use_spot of this V1SessionSpec.  # noqa: E501
        :rtype: bool
        """
        return self._use_spot

    @use_spot.setter
    def use_spot(self, use_spot):
        """Sets the use_spot of this V1SessionSpec.


        :param use_spot: The use_spot of this V1SessionSpec.  # noqa: E501
        :type: bool
        """

        self._use_spot = use_spot

    @property
    def user_id(self):
        """Gets the user_id of this V1SessionSpec.  # noqa: E501


        :return: The user_id of this V1SessionSpec.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this V1SessionSpec.


        :param user_id: The user_id of this V1SessionSpec.  # noqa: E501
        :type: str
        """

        self._user_id = user_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1SessionSpec, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1SessionSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1SessionSpec):
            return True

        return self.to_dict() != other.to_dict()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grid/v1/session.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from grid.protos.grid.v1 import metadata_pb2 as grid_dot_v1_dot_metadata__pb2
from grid.protos.grid.v1 import datastore_pb2 as grid_dot_v1_dot_datastore__pb2
from grid.protos.grid.v1 import resource_pb2 as grid_dot_v1_dot_resource__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='grid/v1/session.proto',
  package='grid.v1',
  syntax='proto3',
  serialized_options=b'Z0github.com/gridai/grid/grid-backend/apis/grid/v1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x15grid/v1/session.proto\x12\x07grid.v1\x1a\x16grid/v1/metadata.proto\x1a\x17grid/v1/datastore.proto\x1a\x16grid/v1/resource.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1fgoogle/api/field_behavior.proto\"\x92\x01\n\x07Session\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x11.grid.v1.MetadataR\x08metadata\x12(\n\x04spec\x18\x02 \x01(\x0b\x32\x14.grid.v1.SessionSpecR\x04spec\x12.\n\x06status\x18\x03 \x01(\x0b\x32\x16.grid.v1.SessionStatusR\x06status\"\xe3\x02\n\x0bSessionSpec\x12\x1d\n\ncluster_id\x18\x01 \x01(\tR\tclusterId\x12\x1d\n\x07user_id\x18\x02 \x01(\tB\x04\xe2\x41\x01\x05R\x06userId\x12:\n\rdesired_state\x18\x03 \x01(\x0e\x32\x15.grid.v1.SessionStateR\x0c\x64\x65siredState\x12#\n\rinstance_type\x18\x04 \x01(\tR\x0cinstanceType\x12\x30\n\tresources\x18\x05 \x01(\x0b\x32\x12.grid.v1.ResourcesR\tresources\x12\x37\n\ndatastores\x18\x06 \x03(\x0b\x32\x17.grid.v1.DatastoreInputR\ndatastores\x12/\n\x10pause_automation\x18\x07 \x01(\x08\x42\x04\xe2\x41\x01\x03R\x0fpauseAutomation\x12\x19\n\x08use_spot\x18\x08 \x01(\x08R\x07useSpot\"\xa3\x04\n\rSessionStatus\x12+\n\x05phase\x18\x01 \x01(\x0e\x32\x15.grid.v1.SessionStateR\x05phase\x12\x17\n\x07ssh_url\x18\x02 \x01(\tR\x06sshUrl\x12&\n\x0fjupyter_lab_url\x18\x03 \x01(\tR\rjupyterLabUrl\x12\x33\n\x16total_run_time_seconds\x18\x04 \x01(\x04R\x13totalRunTimeSeconds\x12\x43\n\x0fstart_timestamp\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0estartTimestamp\x12\x41\n\x0estop_timestamp\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\rstopTimestamp\x12K\n\x13\x64\x65stroyed_timestamp\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x12\x64\x65stroyedTimestamp\x12n\n&last_state_status_transition_timestamp\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\"lastStateStatusTransitionTimestamp\x12*\n\x11jupyter_lab_token\x18\t \x01(\tR\x0fjupyterLabToken*\x98\x01\n\x0cSessionState\x12\x1d\n\x19SESSION_STATE_UNSPECIFIED\x10\x00\x12\x19\n\x15SESSION_STATE_PENDING\x10\x01\x12\x19\n\x15SESSION_STATE_RUNNING\x10\x02\x12\x18\n\x14SESSION_STATE_PAUSED\x10\x03\x12\x19\n\x15SESSION_STATE_DELETED\x10\x04\x42\x32Z0github.com/gridai/grid/grid-backend/apis/grid/v1b\x06proto3'
  ,
  dependencies=[grid_dot_v1_dot_metadata__pb2.DESCRIPTOR,grid_dot_v1_dot_datastore__pb2.DESCRIPTOR,grid_dot_v1_dot_resource__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_api_dot_field__behavior__pb2.DESCRIPTOR,])

_SESSIONSTATE = _descriptor.EnumDescriptor(
  name='SessionState',
  full_name='grid.v1.SessionState',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SESSION_STATE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SESSION_STATE_PENDING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SESSION_STATE_RUNNING', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SESSION_STATE_PAUSED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SESSION_STATE_DELETED', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1231,
  serialized_end=1383,
)
_sym_db.RegisterEnumDescriptor(_SESSIONSTATE)

SessionState = enum_type_wrapper.EnumTypeWrapper(_SESSIONSTATE)
SESSION_STATE_UNSPECIFIED = 0
SESSION_STATE_PENDING = 1
SESSION_STATE_RUNNING = 2
SESSION_STATE_PAUSED = 3
SESSION_STATE_DELETED = 4



_SESSION = _descriptor.Descriptor(
  name='Session',
  full_name='grid.v1.Session',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='metadata', full_name='grid.v1.Session.metadata', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='metadata', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='spec', full_name='grid.v1.Session.spec', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='spec', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='grid.v1.Session.status', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='status', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=174,
  serialized_end=320,
)


_SESSIONSPEC = _descriptor.Descriptor(
  name='SessionSpec',
  full_name='grid.v1.SessionSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='cluster_id', full_name='grid.v1.SessionSpec.cluster_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='clusterId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_id', full_name='grid.v1.SessionSpec.user_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\342A\001\005', json_name='userId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='desired_state', full_name='grid.v1.SessionSpec.desired_state', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='desiredState', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='instance_type', full_name='grid.v1.SessionSpec.instance_type', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='instanceType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resources', full_name='grid.v1.SessionSpec.resources', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resources', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='datastores', full_name='grid.v1.SessionSpec.datastores', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datastores', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pause_automation', full_name='grid.v1.SessionSpec.pause_automation', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\342A\001\003', json_name='pauseAutomation', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='use_spot', full_name='grid.v1.SessionSpec.use_spot', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='useSpot', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=323,
  serialized_end=678,
)


_SESSIONSTATUS = _descriptor.Descriptor(
  name='SessionStatus',
  full_name='grid.v1.SessionStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='phase', full_name='grid.v1.SessionStatus.phase', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='phase', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ssh_url', full_name='grid.v1.SessionStatus.ssh_url', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sshUrl', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='jupyter_lab_url', full_name='grid.v1.SessionStatus.jupyter_lab_url', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='jupyterLabUrl', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='total_run_time_seconds', full_name='grid.v1.SessionStatus.total_run_time_seconds', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='totalRunTimeSeconds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_timestamp', full_name='grid.v1.SessionStatus.start_timestamp', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='startTimestamp', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stop_timestamp', full_name='grid.v1.SessionStatus.stop_timestamp', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='stopTimestamp', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='destroyed_timestamp', full_name='grid.v1.SessionStatus.destroyed_timestamp', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='destroyedTimestamp', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_state_status_transition_timestamp', full_name='grid.v1.SessionStatus.last_state_status_transition_timestamp', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='lastStateStatusTransitionTimestamp', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='jupyter_lab_token', full_name='grid.v1.SessionStatus.jupyter_lab_token', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='jupyterLabToken', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=681,
  serialized_end=1228,
)

_SESSION.fields_by_name['metadata'].message_type = grid_dot_v1_dot_metadata__pb2._METADATA
_SESSION.fields_by_name['spec'].message_type = _SESSIONSPEC
_SESSION.fields_by_name['status'].message_type = _SESSIONSTATUS
_SESSIONSPEC.fields_by_name['desired_state'].enum_type = _SESSIONSTATE
_SESSIONSPEC.fields_by_name['resources'].message_type = grid_dot_v1_dot_resource__pb2._RESOURCES
_SESSIONSPEC.fields_by_name['datastores'].message_type = grid_dot_v1_dot_datastore__pb2._DATASTOREINPUT
_SESSIONSTATUS.fields_by_name['phase'].enum_type = _SESSIONSTATE
_SESSIONSTATUS.fields_by_name['start_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SESSIONSTATUS.fields_by_name['stop_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SESSIONSTATUS.fields_by_name['destroyed_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SESSIONSTATUS.fields_by_name['last_state_status_transition_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
DESCRIPTOR.message_types_by_name['Session'] = _SESSION
DESCRIPTOR.message_types_by_name['SessionSpec'] = _SESSIONSPEC
DESCRIPTOR.message_types_by_name['SessionStatus'] = _SESSIONSTATUS
DESCRIPTOR.enum_types_by_name['SessionState'] = _SESSIONSTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Session = _reflection.GeneratedProtocolMessageType('Session', (_message.Message,), {
  'DESCRIPTOR' : _SESSION,
  '__module__' : 'grid.v1.session_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.Session)
  })
_sym_db.RegisterMessage(Session)

SessionSpec = _reflection.GeneratedProtocolMessageType('SessionSpec', (_message.Message,), {
  'DESCRIPTOR' : _SESSIONSPEC,
  '__module__' : 'grid.v1.session_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.SessionSpec)
  })
_sym_db.RegisterMessage(SessionSpec)

SessionStatus = _reflection.GeneratedProtocolMessageType('SessionStatus', (_message.Message,), {
  'DESCRIPTOR' : _SESSIONSTATUS,
  '__module__' : 'grid.v1.session_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.SessionStatus)
  })
_sym_db.RegisterMessage(SessionStatus)


DESCRIPTOR._options = None
_SESSIONSPEC.fields_by_name['user_id']._options = None
_SESSIONSPEC.fields_by_name['pause_automation']._options = None
# @@protoc_insertion_point(module_scope)



class GridError(Exception):
    """Base Grid exception."""


class APIError(GridError):
    """General Grid API error."""


class ResourceNotFound(GridError):
    """Raised when a resource isn't found on Gri.d"""


class AuthenticationError(Exception):
    """Risen when the user is not authenticated."""


class TrainError(Exception):
    """
    Risen whenever we have an exception during a training
    operation.
    """


class SerializationError(Exception):
    """Risen when the Serializer fails to serialize record."""


class LogLimitExceeded(Exception):
    """Risen when there are more logs than requested"""
    pass



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from grid.protos.grid.v1 import session_service_pb2 as grid_dot_v1_dot_session__service__pb2


class SessionServiceStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.CreateSession = channel.unary_unary(
        '/grid.v1.SessionService/CreateSession',
        request_serializer=grid_dot_v1_dot_session__service__pb2.CreateSessionRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_session__service__pb2.CreateSessionResponse.FromString,
        )
    self.GetSession = channel.unary_unary(
        '/grid.v1.SessionService/GetSession',
        request_serializer=grid_dot_v1_dot_session__service__pb2.GetSessionRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_session__service__pb2.GetSessionResponse.FromString,
        )
    self.ListSessions = channel.unary_unary(
        '/grid.v1.SessionService/ListSessions',
        request_serializer=grid_dot_v1_dot_session__service__pb2.ListSessionsRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_session__service__pb2.ListSessionsResponse.FromString,
        )
    self.UpdateSession = channel.unary_unary(
        '/grid.v1.SessionService/UpdateSession',
        request_serializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionResponse.FromString,
        )
    self.UpdateSessionStatus = channel.unary_unary(
        '/grid.v1.SessionService/UpdateSessionStatus',
        request_serializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionStatusRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionStatusResponse.FromString,
        )
    self.DeleteSession = channel.unary_unary(
        '/grid.v1.SessionService/DeleteSession',
        request_serializer=grid_dot_v1_dot_session__service__pb2.DeleteSessionRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_session__service__pb2.DeleteSessionResponse.FromString,
        )
    self.WatchSessions = channel.unary_stream(
        '/grid.v1.SessionService/WatchSessions',
        request_serializer=grid_dot_v1_dot_session__service__pb2.WatchSessionsRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_session__service__pb2.WatchSessionsResponse.FromString,
        )


class SessionServiceServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def CreateSession(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetSession(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListSessions(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UpdateSession(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UpdateSessionStatus(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DeleteSession(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def WatchSessions(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_SessionServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'CreateSession': grpc.unary_unary_rpc_method_handler(
          servicer.CreateSession,
          request_deserializer=grid_dot_v1_dot_session__service__pb2.CreateSessionRequest.FromString,
          response_serializer=grid_dot_v1_dot_session__service__pb2.CreateSessionResponse.SerializeToString,
      ),
      'GetSession': grpc.unary_unary_rpc_method_handler(
          servicer.GetSession,
          request_deserializer=grid_dot_v1_dot_session__service__pb2.GetSessionRequest.FromString,
          response_serializer=grid_dot_v1_dot_session__service__pb2.GetSessionResponse.SerializeToString,
      ),
      'ListSessions': grpc.unary_unary_rpc_method_handler(
          servicer.ListSessions,
          request_deserializer=grid_dot_v1_dot_session__service__pb2.ListSessionsRequest.FromString,
          response_serializer=grid_dot_v1_dot_session__service__pb2.ListSessionsResponse.SerializeToString,
      ),
      'UpdateSession': grpc.unary_unary_rpc_method_handler(
          servicer.UpdateSession,
          request_deserializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionRequest.FromString,
          response_serializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionResponse.SerializeToString,
      ),
      'UpdateSessionStatus': grpc.unary_unary_rpc_method_handler(
          servicer.UpdateSessionStatus,
          request_deserializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionStatusRequest.FromString,
          response_serializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionStatusResponse.SerializeToString,
      ),
      'DeleteSession': grpc.unary_unary_rpc_method_handler(
          servicer.DeleteSession,
          request_deserializer=grid_dot_v1_dot_session__service__pb2.DeleteSessionRequest.FromString,
          response_serializer=grid_dot_v1_dot_session__service__pb2.DeleteSessionResponse.SerializeToString,
      ),
      'WatchSessions': grpc.unary_stream_rpc_method_handler(
          servicer.WatchSessions,
          request_deserializer=grid_dot_v1_dot_session__service__pb2.WatchSessionsRequest.FromString,
          response_serializer=grid_dot_v1_dot_session__service__pb2.WatchSessionsResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'grid.v1.SessionService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))
